@@ -21,10 +21,8 @@
 #ifndef _RATPOISON_EDITOR_H
 #define _RATPOISON_EDITOR_H 1
 
-typedef enum edit_status edit_status;
 
-enum
-edit_status
+typedef enum edit_status
 {
   EDIT_INSERT,
   EDIT_DELETE,
@@ -33,7 +31,12 @@ edit_status
   EDIT_ABORT,
   EDIT_DONE,
   EDIT_NO_OP
-};
+} edit_status;
+
+/* UTF-8 handling macros */
+#define RP_IS_UTF8_CHAR(c) (defaults.utf8_locale && (c) & 0xC0)
+#define RP_IS_UTF8_START(c) (defaults.utf8_locale && ((c) & 0xC0) == 0xC0)
+#define RP_IS_UTF8_CONT(c) (defaults.utf8_locale && ((c) & 0xC0) == 0x80)
 
 /* Input line functions */
 rp_input_line *input_line_new (char *prompt, char *preinput, int history_id, completion_fn fn);
