@@ -236,10 +236,10 @@ destroy_window (XDestroyWindowEvent *ev)
           if (frame->number == win->scr->current_frame
               && current_screen() == win->scr)
             set_active_frame (frame, 0);
+          /* Since we may have switched windows, call the hook. */
+          if (frame->win_number != EMPTY)
+            hook_run (&rp_switch_win_hook);
         }
-      /* Since we may have switched windows, call the hook. */
-      if (frame->win_number != EMPTY)
-	hook_run (&rp_switch_win_hook);
       withdraw_window (win);
     }
 
@@ -788,8 +788,7 @@ selection_request (XSelectionRequestEvent *rq)
 static void
 selection_clear (void)
 {
-  if (selection.text)
-    free (selection.text);
+  free (selection.text);
   selection.text = NULL;
   selection.len = 0;
 }
@@ -944,7 +943,7 @@ handle_signals (void)
       {
 	      deactivate_screen(&screens[i]);
       }
-      execlp(rp_exec_newwm, rp_exec_newwm, NULL);
+      execlp (rp_exec_newwm, rp_exec_newwm, (char *)NULL);
 
       /* Failed. Clean up. */
       PRINT_ERROR (("exec %s ", rp_exec_newwm));
