@@ -57,25 +57,21 @@ char *
 list_unmanaged_windows (void)
 {
   char *tmp = NULL;
+
   if (unmanaged_window_list)
     {
-      char *tpos;
-      int len = 0;
+      struct sbuf *buf;
       int i;
 
-      for (i = 0; i < num_unmanaged_windows; i++)
-        len += (strlen(unmanaged_window_list[i]) + 1);
-
-      tmp = xmalloc(len + 1);
-      tpos = tmp;
+      buf = sbuf_new (0);
 
       for (i = 0; i < num_unmanaged_windows; i++)
         {
-          sprintf(tpos, "%s\n", unmanaged_window_list[i]);
-          tpos += strlen(unmanaged_window_list[i])+1;
+          sbuf_concat (buf, unmanaged_window_list[i]);
+          sbuf_concat (buf, "\n");
         }
-      tpos--;
-      *tpos = '\0';
+      sbuf_chop (buf);
+      tmp = sbuf_free_struct (buf);
     }
   return tmp;
 }
@@ -210,16 +206,16 @@ get_wmname (Window w)
       Atom type = None;
       unsigned long nitems, bytes_after;
       int format;
-      char *val = NULL;
+      unsigned char *val = NULL;
 
       ret = XGetWindowProperty (dpy, w, _net_wm_name, 0, 40, False,
 				xa_utf8_string, &type, &format, &nitems,
-				&bytes_after, (unsigned char **) &val);
+				&bytes_after, &val);
       /* We have a valid UTF-8 string */
       if (ret == Success && type == xa_utf8_string
 	  && format == 8 && nitems > 0)
 	{
-	  name = xstrdup (val);
+	  name = xstrdup ((char *)val);
 	  XFree (val);
           PRINT_DEBUG (("Fetching window name using _NET_WM_NAME succeeded\n"));
 	  PRINT_DEBUG (("WM_NAME: %s\n", name));
@@ -233,7 +229,7 @@ get_wmname (Window w)
 
   if (XGetWMName (dpy, w, &text_prop) == 0)
     {
-      PRINT_ERROR (("XGetWMName failed\n"));
+      PRINT_DEBUG (("XGetWMName failed\n"));
       return NULL;
     }
 
@@ -524,19 +520,23 @@ unmanaged_window (Window w)
   char *wname;
   int i;
 
-  if (!unmanaged_window_list) return 0;
+  if (!unmanaged_window_list)
+    return 0;
+
+  wname = get_wmname (w);
+  if (!wname)
+    return 0;
 
   for (i = 0; i < num_unmanaged_windows; i++)
     {
-      wname = get_wmname(w);
-      if (!wname) return 0;
-      if (!strcmp(unmanaged_window_list[i], wname))
+      if (!strcmp (unmanaged_window_list[i], wname))
         {
-          free(wname);
+          free (wname);
           return 1;
         }
-      free(wname);
     }
+
+  free (wname);
   return 0;
 }
 
